from typing import Any, Optional

from pydantic import BaseModel, Field


class Image(BaseModel):
    content_type: str
    b64encoded: str


class Answer(BaseModel):
    answer: str
    original_question_uuid: Optional[str]
    actual_question_uuid: Optional[str]
    module: str
    agent_path: str


class Step(BaseModel):
    original_question_uuid: Optional[str]
    actual_question_uuid: Optional[str]
    module: str
    title: str
    value: Optional[str] = None
    agent_path: str
    reason: Optional[str] = None
    timeit: float
    input_nuclia_tokens: Optional[float]
    output_nuclia_tokens: Optional[float]
    error: Optional[str] = None

    def __str__(self) -> str:
        return (
            f"({self.timeit}) {self.module}: {self.title} - {self.value} : "
            f"{self.reason} ({self.input_nuclia_tokens}:{self.output_nuclia_tokens})"
        )


class Chunk(BaseModel):
    chunk_id: str
    title: Optional[str] = None
    source: Optional[str] = None
    text: str
    labels: list[str] = Field(default_factory=list)
    url: list[str] = Field(default_factory=list)
    metadata: Optional[dict[str, Any]] = None


class Prompt(BaseModel):
    prompt: str
    resources: list[str] = Field(default_factory=list)
    links: list[str] = Field(default_factory=list)
    description: Optional[str] = None


class Context(BaseModel):
    original_question_uuid: Optional[str]
    actual_question_uuid: Optional[str]
    question: str
    chunks: list[Chunk] = Field(default_factory=list)
    images: dict[str, Image] = Field(default_factory=dict)
    prompts: list[Prompt] = Field(default_factory=list)
    structured: list[str] = Field(default_factory=list)
    source: str
    agent: str
    # XXX: This is not actually a summary, but an answer attempt for now!
    summary: str = Field(
        default="",
        description="Partial or full answer to the question,"
        " generated by the context validation step inside a context agent.",
    )
    agent_id: str = ""
    title: Optional[str] = None
    missing: Optional[str] = None
    citations: Optional[list[str]] = Field(
        default=None,
        description="list of chunk IDs that were considered relevant in the context validation step.",
    )



import uuid
from enum import Enum
from typing import Any, Optional

from pydantic import BaseModel, Field

from nuclia_models.agent.memory import Answer, Context, Step

# Models used between client and API for itneraction requests/responses/streams


class Operation(int, Enum):
    START = 0
    STOP = 1


class InteractionOperation(int, Enum):
    QUESTION = 0
    QUIT = 1
    USER_RESPONSE = 2


class AnswerOperation(int, Enum):
    ANSWER = 0
    START = 2
    DONE = 3
    ERROR = 4
    AGENT_REQUEST = 5


class InteractionRequest(BaseModel):
    question: str
    headers: dict[str, str] = {}
    operation: InteractionOperation = InteractionOperation.QUESTION


class UserToAgentInteraction(BaseModel):
    """Sends a user response to an agent request"""

    request_id: str
    response: str
    operation: InteractionOperation = InteractionOperation.USER_RESPONSE


class ARAGException(BaseModel):
    detail: str


class ValidationFeedbackSchema(BaseModel):
    call_tool: bool


class PromptFeedbackSchema(BaseModel):
    prompt_id: str
    data: Any


class Feedback(BaseModel):
    request_id: str
    feedback_id: str = Field(default_factory=lambda: uuid.uuid4().hex)
    question: str
    module: str
    agent_id: str
    data: Any
    timeout_ms: int = 10_000
    response_schema: Any


class AragAnswer(BaseModel):
    exception: Optional[ARAGException] = None
    answer: Optional[str] = None
    agent_request: Optional[str] = None
    generated_text: Optional[str] = None
    step: Optional[Step] = None
    possible_answer: Optional[Answer] = None
    context: Optional[Context] = None
    operation: AnswerOperation = AnswerOperation.ANSWER
    seqid: Optional[int] = None
    original_question_uuid: Optional[str] = None
    actual_question_uuid: Optional[str] = None
    feedback: Optional[Feedback] = None



from enum import Enum


class TextFormat(Enum):
    PLAIN = "PLAIN"
    HTML = "HTML"
    RST = "RST"
    MARKDOWN = "MARKDOWN"
    JSON = "JSON"
    KEEP_MARKDOWN = "KEEP_MARKDOWN"
    JSONL = "JSONL"
    PLAIN_BLANKLINE_SPLIT = "PLAIN_BLANKLINE_SPLIT"
